/* (c) Copyright 1998 International Game Technology */

/*
   $Workfile:   Debug.cpp  $
   $Archive:   P:/PIG/archives/Common/Debug.cpv  $
   $Revision:   1.3  $
   $Date:   Nov 09 1999 15:04:48  $
   $Author:   slemay  $

   Purpose:
      Defines the class Debug.
      The idea is to get the whole debug functionality in this single .hpp file to make
      it simple on the user.  When __DEBUG__ is #defined it increases the object code
      size because each #include of this file creates another debug object.  However,
      when __DEBUG__ is NOT #defined the compiler has been seen to completely optimize
      out all references to debug.

   Modifications:
      Author          Date       Explanation
      --------------- ---------- ------------
      Dwayne Nelson   Oct 28, 98 New
      Dwayne Nelson   Jun 09, 99 Added buffering the the DebugStreambuf to help eliminate the problem
                                 where lines in the debug log clobber over each other.
      Dwayne Nelson   Jul 30, 99 Removed stream manipulators that shouldn't be here because they
                                 aren't debug specific.
      LeMay, S.       Sep 30, 99 Modified to work with QNX Neutrino
*/

#ifndef __DEBUG_HPP__
#include "Debug.hpp"
#endif

#ifdef __DEBUG__

#if defined (__QNXNTO__)
   #ifndef __NEUTRINO_H_INCLUDED
   #include <sys/neutrino.h>
   #endif
   #ifndef _FCNTL_H_INCLUDED
   extern "C" {
      #include <fcntl.h>
   }
   #endif
#elif defined (__QNX__)
   #ifndef __KERNEL_H_INCLUDED
   #include <sys/kernel.h>
   #endif
   #ifndef __NAME_H_INCLUDED
   #include <sys/name.h>
   #endif
   #ifndef __SENDMX_H_INCLUDED
   #include <sys/sendmx.h>
   #endif
#endif

#ifndef _UNISTD_H_INCLUDED
#include <unistd.h>
#endif
#ifndef __DEBUGINTERFACE_HPP__
#include "DebugInterface.hpp"
#endif

// DebugMonitor
// This is the class that is the debug client.
// This class talks to the debug process.  It doesn't use or derive from
// Sender.  This is intentional so that debug functionality isn't dependent
// on other functionality.  This ensures that the debug functionality can
// be used anywhere in the code base.
class DebugMonitor
{
   public:
      DebugMonitor (void)
         : pid (-1) {}
      ~DebugMonitor (void)
      	{
      		#if defined (__QNXNTO__)
      		if (pid >= 0)
      		   close (pid);
      		pid = -1;
      	   #endif
      	}   
      
      void print (const char * buf, int len)
         {
            #if defined (__QNXNTO__)
               if (pid == -1)
                  pid = open (DEBUG_REGISTERED_NAME, O_RDWR);
            #elif defined (__QNX__)
               if (pid == -1)
                  pid = qnx_name_locate (0, DEBUG_REGISTERED_NAME ,0, 0);
            #endif

            if (pid != getpid () && pid != -1)
            {
               DebugMessagePrint message (Version (1,0));
               message.msg_length = len;

               #if defined (__QNXNTO__)
                  iov_t send_mx [2];
                  SETIOV (send_mx + 0, &message, sizeof (message));
                  SETIOV (send_mx + 1, buf, len);
                  
                  // If the send fails, clear the pid
                  if (MsgSendv(pid, send_mx, ARRAY_ELEMENTS (send_mx), 0, 0) == -1)
               	   pid = -1;
               #elif defined (__QNX__)
                  _mxfer_entry send_mx [2];
                  _setmx (send_mx + 0, &message, sizeof (message));
                  _setmx (send_mx + 1, buf, len);
                  
                  // If the send fails, clear the pid
                  if (Sendmx (pid, ARRAY_ELEMENTS (send_mx), 0, send_mx, 0) == -1)
                     pid = -1;
               #endif
            }
         }
   protected:
      int pid;      // Connection Id.
};


#if defined (__QNXNTO__)

   // DebugStreambuf
   // The streambuf used by DebugStream
   // This class basically takes all stream information and sends it to the debug
   // process by using the DebugMonitor class.
   class DebugStreambuf : public streambuf
   {
      public:
         DebugStreambuf (void)
            {
				   // Set the buffer pointers to our buffer
               setp (buffer, buffer + BUFFER_SIZE);
            }

             // Purpose: Virtual function called when the put buffer gets full
         virtual int overflow (int ch = EOF)
            {
				   // Send the information to the Debug process
               sender.print (pbase (), out_waiting ());
				   // Reset the put pointers
               setp (pbase(), pbase() + BUFFER_SIZE);
               return streambuf::overflow (ch);
            }

         // Purpose: Virtual function that is called to flush/synch up the buffer with the actual device
         //	NOTE: The main reason for overriding this is when endl is sent to the stream this 
         //	method is called.  Therefore we can flush out the debug info to the debug process
         virtual int sync (void)
            {
				   // Flush our buffer
               overflow();          						
               return streambuf::sync ();
            }

      protected:
		   enum {BUFFER_SIZE = 512,};
		   char buffer [BUFFER_SIZE];   // The buffer
         DebugMonitor sender;         // The guy who actually talks to the debug process
      
         int out_waiting (void) const
            { return (pptr () - pbase ()) / sizeof (char); }
};

#elif defined (__QNX__)

   // DebugStreambuf
   // The streambuf used by DebugStream
   // This class basically takes all stream information and sends it to the debug
   // process by using the DebugMonitor class.
   class DebugStreambuf : public streambuf
   {
      public:
         DebugStreambuf (void)
            {
               unbuffered(0); // Set to not-unbuffered = Buffered
               allocate();    // Allocate memory
            }

         // Purpose: Virtual function called to allocate our buffers
         int doallocate (void)
            {
               setbuf (buffer, BUFFER_SIZE);       // Give the stream a buffer for buffering
               setp (buffer, buffer+BUFFER_SIZE);  // Set the put pointers
               return __NOT_EOF;            
            }

         // Purpose: Virtual function called when the put buffer gets full
         int overflow (int ch = EOF)
            {
               sender.print (pbase(), out_waiting()); // Send the information to Debug process
               setp (pbase(), pbase()+BUFFER_SIZE);   // Reset the put pointers
               return __NOT_EOF;
            }

         // Purpose: Virtual function that is called to flush/synch up the buffer with the actual device
         int sync (void)
            {
               streambuf::sync();   // Default
               overflow();          // Flush our buffer
               return __NOT_EOF;
            }

         // Purpose: Virtual function that will get called when the get buffer is empty - We'll never see this
         int underflow (void)
            {
               // This method should never get called
               return __NOT_EOF;
            }

      protected:
         enum { BUFFER_SIZE = 512 };
         char buffer [BUFFER_SIZE];  // The buffer
         DebugMonitor sender;        // The guy who actually talks to the debug process
   };

#endif

static DebugStreambuf debug_buf;  // Create a streambuf for the DebugStream to use

DebugStream::DebugStream (void)
	: ostream (&debug_buf) {}

DebugStream real_debug;

#endif

